--- ori/asterisk-1.6.2.1/channels/chan_sip.c	2010-01-07 22:17:43.000000000 +0100
+++ asterisk-1.6.2.1/channels/chan_sip.c	2010-01-28 12:51:55.000000000 +0100
@@ -1825,6 +1825,7 @@ struct sip_peer {
 		AST_STRING_FIELD(context);		/*!< Default context for incoming calls */
 		AST_STRING_FIELD(subscribecontext);	/*!< Default context for subscriptions */
 		AST_STRING_FIELD(username);		/*!< Temporary username until registration */ 
+		AST_STRING_FIELD(challengeuser);	/*!< Username used in challenge authentication */ 
 		AST_STRING_FIELD(accountcode);		/*!< Account code */
 		AST_STRING_FIELD(tohost);		/*!< If not dynamic, IP address */
 		AST_STRING_FIELD(regexten); 		/*!< Extension to register (if regcontext is used) */
@@ -12984,7 +12985,8 @@ static enum check_auth_result register_v
 			ast_copy_flags(&p->flags[0], &peer->flags[0], SIP_NAT);
 			if (ast_test_flag(&p->flags[1], SIP_PAGE2_REGISTERTRYING))
 				transmit_response(p, "100 Trying", req);
-			if (!(res = check_auth(p, req, peer->name, peer->secret, peer->md5secret, SIP_REGISTER, uri, XMIT_UNRELIABLE, req->ignore))) {
+			ast_log(LOG_NOTICE, "Tryng athenticating peer '%s' using challengeuser: '%s'\n", peer->name, peer->challengeuser);
+			if (!(res = check_auth(p, req, peer->challengeuser, peer->secret, peer->md5secret, SIP_REGISTER, uri, XMIT_UNRELIABLE, req->ignore))) {
 				if (sip_cancel_destroy(p))
 					ast_log(LOG_WARNING, "Unable to cancel SIP destruction.  Expect bad things.\n");
 
@@ -13979,7 +13981,7 @@ static enum check_auth_result check_peer
 		ast_string_field_set(p, peersecret, NULL);
 		ast_string_field_set(p, peermd5secret, NULL);
 	}
-	if (!(res = check_auth(p, req, peer->name, p->peersecret, p->peermd5secret, sipmethod, uri2, reliable, req->ignore))) {
+	if (!(res = check_auth(p, req, peer->challengeuser, p->peersecret, p->peermd5secret, sipmethod, uri2, reliable, req->ignore))) {
 		ast_copy_flags(&p->flags[0], &peer->flags[0], SIP_FLAGS_TO_COPY);
 		ast_copy_flags(&p->flags[1], &peer->flags[1], SIP_PAGE2_FLAGS_TO_COPY);
 		/* If we have a call limit, set flag */
@@ -15359,6 +15361,7 @@ static char *_sip_show_peer(int type, in
 		}
 		ast_cli(fd, "  Secret       : %s\n", ast_strlen_zero(peer->secret)?"<Not set>":"<Set>");
 		ast_cli(fd, "  MD5Secret    : %s\n", ast_strlen_zero(peer->md5secret)?"<Not set>":"<Set>");
+		ast_cli(fd, "  Challen.User : %s\n", peer->challengeuser);
 		ast_cli(fd, "  Remote Secret: %s\n", ast_strlen_zero(peer->remotesecret)?"<Not set>":"<Set>");
 		for (auth = peer->auth; auth; auth = auth->next) {
 			ast_cli(fd, "  Realm-auth   : Realm %-15.15s User %-10.20s ", auth->realm, auth->username);
@@ -23679,8 +23682,10 @@ static struct sip_peer *build_peer(const
 		set_peer_defaults(peer);	/* Set peer defaults */
 		peer->type = 0;
 	}
-	if (!found && name)
+	if (!found && name) {
 		ast_copy_string(peer->name, name, sizeof(peer->name));
+		ast_string_field_set(peer, challengeuser, name);	/* Set default challengeuser*/
+	}
 
 	/* If we have channel variables, remove them (reload) */
 	if (peer->chanvars) {
@@ -23745,6 +23750,8 @@ static struct sip_peer *build_peer(const
 				ast_string_field_set(peer, secret, v->value);
 			} else if (!strcasecmp(v->name, "md5secret")) {
 				ast_string_field_set(peer, md5secret, v->value);
+			} else if (!strcasecmp(v->name, "challengeuser")) {
+				ast_string_field_set(peer, challengeuser, v->value);	
 			} else if (!strcasecmp(v->name, "auth")) {
 				peer->auth = add_realm_authentication(peer->auth, v->value, v->lineno);
 			} else if (!strcasecmp(v->name, "callerid")) {
